LoadPackage("atlasrep");
LoadPackage("io");

checkD_random := function(gr, x, n)
  local sg, i, t, y;
  Print("Trying with n=", n, "\n");
  for i in [1..n] do
  	t := Random(gr);
    y := t*x*Inverse(t);
    if not (x*y)^2 = (y*x)^2 then 
      sg := Subgroup(gr, [x,y]);
      if IsConjugate(sg, x, y) = false then
        return [x,y];
      fi;
    fi;
  od;
  return fail;
end;

checkD := function(gr, c)
  local sg, x, y;
  x := Representative(c);
  for y in c do
    if not (x*y)^2 = (y*x)^2 then 
      sg := Subgroup(gr, [x,y]);
      if IsConjugate(sg, x, y) = false then
        return [x,y];
      fi;
    fi;
  od;
  return fail;
end;

x := [];
g := [];
n := 0;

name := "He.2";

LogTo(Concatenation(name, ".log"));

M := 12; 

gr := AtlasGroup(name);
cc := ConjugacyClasses(gr);
ct := CharacterTable(name);

Print("Group: ", name, "\n");
Print("Number of maximal subgroups: ", M, "\n");

ns := ClassPositionsOfNormalSubgroups(ct);
cl := Filtered([1..NrConjugacyClasses(ct)], x->not x in ns[2]);
Print("Conjugacy classes to study:\n");
for x in cl do
  Print(ClassNames(ct, "atlas")[x], ", ");
od;
Print("\n");
all := List(cl, x->[OrdersClassRepresentatives(ct)[x], SizesConjugacyClasses(ct)[x], SizesCentralizers(ct)[x]]);
tmp := List(cl, x->[OrdersClassRepresentatives(ct)[x], SizesConjugacyClasses(ct)[x], SizesCentralizers(ct)[x]]);
Display(tmp);
Print("==\n");

# In this example it is impossible to connect the group and the character table

bg := BaseOfGroup(gr);
Print("Base of Group: ", bg, "\n--\n");

f := IO_File(Concatenation(name, ".rec"), "w");
IO_WriteLine(f,"### The format for classinfo is [ r, s, order, comment ]");
IO_WriteLine(f,"data := rec(");
IO_WriteLine(f,"  base := ", bg, ",");
IO_WriteLine(f,"  group := \"", name, "\",");
IO_WriteLine(f,"  comments := \"Maximal subgroups\",");
IO_WriteLine(f,"  classinfo := [");

rem := [];

M := 12; 

for i in [1..M] do

  _gr := Group(AtlasGenerators(name, 1, M+1-i).generators);
  _cc := ConjugacyClasses(_gr);

  # For  an  ordinary character table <tbl> with known underlying group $G$,
  # IdentificationOfConjugacyClasses returns a list of positive integers that
  # contains at position i the position of the i-th conjugacy class of <tbl>
  # in the list ConjugacyClasses(G).
  for c in _cc do
    g := Representative(c);
    n := Position(List(cc, x->g in x), true);
    if not n in rem and [Order(g), Size(cc[n]), Size(gr)/Size(cc[n])] in tmp then
      Print("Checking (maximal subgroup ", M+1-i, "), ");
      Print(" representative of order ", Order(g), "\n");
      rs := checkD(_gr, c); 
      if not rs = fail then
        Print(" Conjugacy class of size: ", Size(c), "\n");
        Print(" Centralizer of order: ", Size(gr)/Size(c), "\n");
      	Print("r:=", OnTuples(bg, rs[1]), ";\n");
      	Print("s:=", OnTuples(bg, rs[2]), ";\n");
        IO_WriteLine(f, "    [ ", OnTuples(bg, rs[1]), ", ", OnTuples(bg, rs[2]), ", ", Order(g), ", \"D\" ],");
        Print("Removed!\n");
        Remove(tmp, Position(tmp, [Order(g), Size(cc[n]), Size(gr)/Size(cc[n])])); 
        Add(rem, n);
        Print("--\n");
      fi;
    fi;
  od;
od;

diff := Difference(all, tmp);

Print("==\nSummary:\n");
Print("Maybe not of type D: ", tmp, "\n");
Print("==\n");

IO_WriteLine(f, "  ]");
IO_WriteLine(f, ");");
IO_Flush(f);
IO_Close(f);


